-- Initialize local namespace 
local addonName, addon = ...
local _G = _G

BlizzardInterfaceDisplayManager = { }
BlizzardInterfaceDisplayManager_Saved = { }
BlizzardInterfaceDisplayManager_Saved.FadeBlizzardUI = false;

-- on ADDON_LOADED for this addon
local function onAddonLoaded()
	BlizzardInterfaceDisplayManager.FadeBlizzardUI = Classy.Observable.new(BlizzardInterfaceDisplayManager_Saved.FadeBlizzardUI)
	BlizzardInterfaceDisplayManager.FadeBlizzardUI:subscribe(function (newValue)
		BlizzardInterfaceDisplayManager_Saved.FadeBlizzardUI = newValue;
		local fadeit = (newValue and (DoIt.Events.isPlayerOnTaxi:get() or UnitIsDeadOrGhost("player")));
		addon.MainMenuBarFade(fadeit)
	end)
end
DoIt.RegisterModule(addonName, addon, onAddonLoaded);

local allClassic = function (alpha)
	ReputationWatchBar:SetAlpha(alpha)
	MainMenuExpBar:SetAlpha(alpha)
	StanceBarFrame:SetAlpha(alpha)
	PetActionBarFrame:SetAlpha(alpha)
	MainMenuBarArtFrame:SetAlpha(alpha)
end

local allRetail = function (alpha)
	PossessBarFrame:SetAlpha(alpha)
	MicroButtonAndBagsBar:SetAlpha(alpha)
	StanceBarFrame:SetAlpha(alpha)
	PetActionBarFrame:SetAlpha(alpha)
	MainMenuBarArtFrame:SetAlpha(alpha)
end

local function setAllAlphas(alpha)
	if (DoIt.IsClassic) then
		allClassic(alpha);
	else
		allRetail(alpha);
	end
end

---
--- Fades Blizzards main menu bar and it's related components
---
function addon.MainMenuBarFade(state)
    state = not state;
	if (not state) then
		if (MainMenuBarArtFrame:GetAlpha() ~= 0.25) then
			setAllAlphas(0.25);
		end
	else
		if (MainMenuBarArtFrame:GetAlpha() ~= 1) then
			setAllAlphas(1);
		end
	end
end

---
--- Shows/Hides Blizzards main menu bar and it's related components
---
function addon.ShowMainMenuBar(show)
	if (not show) then
		if (not DoIt.IsClassic) then
			PossessBarFrame:Hide()
			MicroButtonAndBagsBar:Hide()
		end
		StanceBarFrame:Hide()
		PetActionBarFrame:Hide()
		MainMenuBarArtFrame:Hide()
		if (DoIt.IsClassic) then
			MainMenuBarPerformanceBar:Hide();
		end
	else
		if (GetNumShapeshiftForms() > 0) then
			StanceBarFrame:Show()
		end
		if (HasPetUI()) then 
			PetActionBarFrame:Show()
		end
		MainMenuBarArtFrame:Show()
		if (not DoIt.IsClassic) then
			MicroButtonAndBagsBar:Show()
		else
			MainMenuBarPerformanceBar:Show();
		end
	end
end

---
--- This is used to show the main menu bar for vehicles, when it is otherwise hidden, or you will not see the
--- a vehicle's action bars. [VERIFY THIS]
---
function addon.ManageMainMenuBar(show)
	local areAnyShown = StanceBarFrame:IsShown() or PetActionBarFrame:IsShown() or MainMenuBarArtFrame:IsShown() or (MicroButtonAndBagsBar and MicroButtonAndBagsBar:IsShown());
	local areAllHidden = not (StanceBarFrame:IsShown() or PetActionBarFrame:IsShown() or MainMenuBarArtFrame:IsShown() or (MicroButtonAndBagsBar and MicroButtonAndBagsBar:IsShown()));
	if (not DoIt.PlayerInVehicle() and areAnyShown) then
		addon.ShowMainMenuBar(false);
	elseif (not DoIt.PlayerInVehicle() and areAllHidden) then
		addon.ShowMainMenuBar(true);
	end
end

local fade = function()
    if (not BlizzardInterfaceDisplayManager_Saved.FadeBlizzardUI) then
        addon.MainMenuBarFade(false)
    else
        addon.MainMenuBarFade(true)
    end
end
local restore = function() addon.MainMenuBarFade(false) end
DoIt.Events.onTaxiStart:subscribe(fade)
DoIt.Events.onTaxiEnd:subscribe(restore)
DoIt.Events:subscribe("PLAYER_DEAD", fade)
DoIt.Events:subscribe("PLAYER_UNGHOST", restore)
DoIt.Events:subscribe("PLAYER_ALIVE", restore)

-- Exports
BlizzardInterfaceDisplayManager.ShowMainMenuBar = addon.ShowMainMenuBar
BlizzardInterfaceDisplayManager.ManageMainMenuBar = addon.ManageMainMenuBar